

#include "config.h"

struct request
{
	char buff[MAX_BUFF];
	int buff_size;
	char filename[MAX_BUFF];
	int parsing_ok;
};

struct response
{
	int code;
	char * content_type;
	int fd;
	int isdir;
};


int read_request (int fd, struct request * rq)
{
	/* Arguments:
	 * - fd: the socket containing the client connection
	 * - rq: we will store here the HTTP request received from the client
	 *
	 * Objective:
	 * - read data from the client until you have read an entire HTTP request,
	 *   that is, read data until you receive the sequence
	 *   "\r\n\r\n", which marks the end of the request
	 * - store the request in rq->buff and set rq->buff_size accordingly
	 *
	 * - return -1 if there is reading problems; else return 0
	 */

	return 0;
}

void parse_request (struct request * rq)
{
	/* Arguments:
	 * - rq: a structure describing the client request; only the field rq->buff
	 *   and rq->buff_size have been initialized and contains the HTTP request
	 *   sent by the client; rq->buff contains rq->buff_size bytes
	 *
	 * Objective:
	 * - parse rq->buff, extract the path of the file requested by the client and
	 *   store it in rq->filename (make it be a null-terminated string)
	 * - set rq->parsing_ok to true iff this function succeeds in extracting a
	 *   file name from rq->buff
	 *
	 */
}

void build_response (struct request * rq, struct response * re)
{
	/* Arguments:
	 * - rq: a structure describing the client request
	 * - re: output argument, we will initialize in this function
	 *
	 * Objective:
	 *
	 * - initialize the structure "re":
	 *
	 * - set re->code to the appropriate response code:
	 *   200 -> file or directory found
	 *   404 -> file or directory not found
	 *   400 -> Bad Request (ie, rq->parsing_ok is false)
	 *   500 -> method not implemented (POST, HEAD, and other unexpected things)
	 *   etc.
	 * - set re->content_type for a couple of common file extensions:
	 *   .txt -> text/plain
	 *   .pdf -> application/pdf
	 *   .png -> image/png
	 *   etc
	 *
	 * - set re->fd to the file descriptor describing the resource that we will
	 *   send to the client
	 *
	 * - set re->isdir to true iff rq->filename points to a directory (we will
	 *   use this field in the serve() function)
	 */
}

void write_response_headers (int fd, struct response * re)
{
	/* Arguments:
	 * - fd: socket with the client connection
	 * - re: a structure describing the headers we should send to the client
	 *
	 * Objective:
	 * - sending the HTTP response to the client, only the response and the
	 *   headers, not the body
	 * - send the header "Server: mini-http/0.1"
	 * - send the header "Content-Type" only if re->content_type != NULL
	 *
	 */
}

void write_response_body_file (int fd, int filefd)
{
	/* Arguments:
	 * - fd: socket with the client connection
	 * - filefd: a file descriptor associated to a regular file
	 *
	 * Objective:
	 * - copy (send) the file attached to "filefd" to the client
	 *
	 */
}

void write_response_body_dir (int fd, int filefd)
{
	/* Arguments:
	 * - fd: socket with the client connection
	 * - filefd: a file descriptor associated to a directory
	 *
	 * Objective:
	 * - using the opendir(3) and readdir(3) functions, read the directory
	 *   associated to "filefd" and generate an HTML page that contains HTML
	 *   links to the files stored in the directory
	 * - send this HTML document to the client, writing it to fd
	 *
	 */
}

void serve (int fd)
{
	/*
	 * - read a request from fd (printing errors and aborting if the reading
	 *   fails)
	 * - parse the resquest and build a response
	 * - send the HTTP response headers to the client
	 * - just after, send the file (or directory) to the client
	 */

	struct request rq;
	struct response re;
	int ret;

	ret = read_request (fd, &rq);
	if (ret < 0)
	{
		printf ("mini-httpd: errors reading request, aborting");
		return;
	}

	parse_request (&rq);
	build_response (&rq, &re);
	write_response_headers (fd, &re);
	if (re.isdir)
	{
		write_response_body_dir (fd, re.fd);
	}
	else
	{
		write_response_body_file (fd, re.fd);
	}
}

